import getopt
import sys
from string import Template
import parser

#========================================================================

class ConfigError(Exception):
#----------------------------------------------------------------------
    def __init__(self, value): self.value = value
#----------------------------------------------------------------------
    def __str__(self): return repr(self.value)

#========================================================================

class Config:
#----------------------------------------------------------------------
    def __init__(self, verif=None, nones=None, verbose=0):
        self.comment, self.include, self.sameline, self.variable = ('#', '@', '\\', '$')
        self.vars = dict()
        self.verif, self.nones, self.verbose = (verif, nones, verbose)
#----------------------------------------------------------------------
    def nonevalues(self):
        if self.nones is None: return
        for v in self.nones:
            for k in self.cfg.keys():
                if self.cfg[k] is v: self.cfg[k] = None
#----------------------------------------------------------------------
    def set(self, indict, path):
        try: f = open(path, "r")
        except IOError, e: self.err("Unable to open file %s" % path)

        d = dict()
        l, ll = ("", "")
        try:
            for l in f:
                l0 = l.strip()
                if len(l0) == 0 or l0[0] == self.comment: continue
                ll += l0.replace(self.sameline, ' ')
                if l0[-1] == self.sameline: continue
                if ll[0] == self.include: self.set(indict, ll[1:])
                else:
                    name, value = ll.split("=")
                    if name[0] == self.variable: self.vars[name[1:].strip()] = self.evaluate(value.strip(), path)
                    else: d[name.strip()] = value.strip()
                ll = ""
            f.close()
        except IOError, e: self.err("Unable to read file %s" % path)
        except StandardError, e: self.err("Error in file %s: %s" % (path, l))

        self.update(indict, d, "file %s" % path)
#----------------------------------------------------------------------
    def evaluate(self, value, msg):
        try: return Template(value).substitute(self.vars).strip()
        except KeyError, e: self.err("%s: Undefined variable used in %s" % (value, msg))
#----------------------------------------------------------------------
    def update(self, indict, dico, msg):
        for k in dico.keys():
            if not indict.has_key(k): self.info("unused key %s in %s" % (k, msg))
        for k in indict.keys():
            dtype, st, default = indict[k]
            if dico.has_key(k):
                # First evluate variables
                dicoval = self.evaluate(dico[k].strip(), msg)
                # Then convert values with type
                try:
                    if len(dicoval) != 0 and dicoval[0] == '[':
                        # We have a list
                        if dicoval[-1] != ']': self.err("%s : Unbalanced delimitors in %s" % (k, msg))
                        else:
                            values = dicoval[1:-1].split(",")
                            #print values[0]
                            if not isinstance(self.cfg[k], list):
                                self.cfg[k] = values
                            for i in range(len(values)):
                                val = values[i].strip()
                                if val is '': val = default
                                else: val = dtype(val)
                                if i < len(self.cfg[k]): self.cfg[k][i] = val
                                else: self.cfg[k].append(val)
#                                if val is not '':
#                                    if i < len(self.cfg[k]): self.cfg[k][i] = dtype(val)
#                                    else: self.cfg[k].append(dtype(val))
                    else: self.cfg[k] = dtype(dicoval)
                except ValueError, e: self.err("%s : Unable to convert %s to %s" % (k, dico[k], dtype))
#----------------------------------------------------------------------
#    def get(self, indict, nones=None):
    def get(self, indict):
        
        self.cfg = dict()
        self.errors, self.errors, optstr = (False, False, "")
        for k in indict.keys():
            self.cfg[k] = indict[k][2]
            optstr += indict[k][1]
        optlist, args = getopt.getopt(sys.argv[1:], optstr)

# Parse command files if some
        if len(args) != 0:
            for i in range(len(args)): self.set(indict, args[i])

# Now parse command line
        dopt = dict()
        for opt in optlist:
            for k in indict.keys():
                if opt[0][1] == indict[k][1][0]:
                    if opt[1] is '': dopt[k] = '1'
                    else: dopt[k] = opt[1]
                    break
        self.update(indict, dopt, "command line")

# Dictionnary local validation
        for k in self.cfg.keys():
            if isinstance(self.cfg[k], list):
                for kk in self.cfg[k]:
                    if kk is None:
                        self.err("%s : at least one required value is not present in configuration" % k)
            elif self.cfg[k] is None:
                self.err("%s : required value not present in configuration" % k)

        self.nonevalues()

# Dictionnary user validation if needed
        if self.verif is not None:
            ok, msg = self.verif(self.cfg)
            if not ok: self.err(msg)
                
# Check if errors appeared
        if self.errors: raise ConfigError("Configuration Errors")
        
        return self.cfg
#----------------------------------------------------------------------
    def err(self, msg):
        self.errors = True
        self.information("Error", msg)
#----------------------------------------------------------------------
    def warn(self, msg):
        self.warns = True
        self.information("Warning", msg)
#----------------------------------------------------------------------
    def info(self, msg):
        if self.verbose: self.information("Information", msg)
#----------------------------------------------------------------------
    def information(self, errtype, msg):
        if not isinstance(msg, list): msgs = [msg]
        else: msgs = msg
        for m in msgs:
            print "***Config " + errtype + "*** " + m
#----------------------------------------------------------------------
    def display(self, prefix):
        print "\n" + prefix + "Configuration:"
        keys = self.cfg.keys()
        keys.sort()
        for k in keys: print prefix + "  " + k + " : ", self.cfg[k]
        print ""
