#! /usr/bin/env python
# -*- coding: latin-1 -*-

import os
from os.path import split, splitext, exists, join
from numpy import *
from struct import *

import numpy



def GetDataType(typ):
    if   typ == "C" : return 's'
    elif typ == "I1": return 'B'
    elif typ == "I2": return 'h'
    elif typ == "i2": return 'H'    
    #elif typ == "I4": return 'i'
    elif typ == "I4": return 'I'
    elif typ == "F4": return 'f'
    elif typ == "F8": return 'd'
    else            : return None
    
def GetNumpyType(typ):
    #if   typ == "I1": return  numpy.uint8
    #elif typ == "I2": return  numpy.int16
    #elif typ == "I4": return  numpy.int32
    #elif typ == "F4": return  numpy.float32
    #elif typ == "F8": return  numpy.float64
    if   typ == "B": return  numpy.uint8
    elif typ == "h": return  numpy.int16
    elif typ == "H": return  numpy.uint16    
    elif typ == "i": return  numpy.int32
    elif typ == "I": return  numpy.uint32
    else            : return None

def GetRecordDesc(rec_pth):
    f = open(rec_pth, "r")
    lines = f.read().splitlines()
    rec_dsc = []
    for l in lines:
        if len(l) == 0: continue
        if l[0] == "#": continue
        nam, typ, lth, dim = l.split()
        dims = map(int, dim.split(","))
        if typ != "C": typ += lth
        rec_dsc.append([nam, GetDataType(typ), int(lth), dims, prod(array(dims))])
    f.close()
    return rec_dsc

def GetRecord(f, dsc, bigend, raw=0):
    rec_flds = []
    for d in dsc:
        if bigend: fmt = ">"
        else: fmt = "<"
        if d[1] == 's':
            for i in range(int(d[4])): fmt += "%d%s" % (d[2], d[1])
        else:
            fmt += "%d%s" % (d[4], d[1])
        fld = unpack(fmt, f.read(d[2]*d[4]))
        if raw: rec_flds.append(fld)
        elif d[4] == 1: rec_flds.append(fld[0])
        else: rec_flds.append(reshape(array(fld), d[3]))
    return rec_flds

def PrintRecord(flds, dsc):
    for d in range(len(dsc)):
        print dsc[d],' : '
        print flds[d]
    
def GetSdsIndex(dsc, sdname):
    for i in range(len(dsc)):
        if dsc[i][0] == sdname: return i
    return -1

def GetSdsdim(dsc, sdname):
    for i in range(len(dsc)):
        if dsc[i][0] == sdname: return dsc[i][3]
    return -1

def GetSdstype(dsc, sdname):
    for i in range(len(dsc)):
        #if dsc[i][0] == sdname: return GetNumpyType(dsc[i][1].capitalize()+str(dsc[i][2]))
        if dsc[i][0] == sdname: return GetNumpyType(dsc[i][1])

    return -1

def MakeDataArray(rec_flds, isds, dims, sds_type):
    
    rec_arr = zeros([len(rec_flds)] + dims, dtype=sds_type)
    if rec_arr.ndim == 3 :
        for i in range(len(rec_flds)):
            rec_arr[i,:,:] = array(rec_flds[i][isds])
    if rec_arr.ndim == 2 :
        for i in range(len(rec_flds)):
            rec_arr[i,:] = array(rec_flds[i][isds])
    return rec_arr
    
def MakeData(records,rec_dsc,sds_name):
    sds_dim = GetSdsdim(rec_dsc, sds_name)
    sds_type=GetSdstype(rec_dsc, sds_name)    
    # Fabrication des donnes  retourner TBD
    data_SDS=MakeDataArray(records,GetSdsIndex(rec_dsc,sds_name),sds_dim,sds_type)
    #print "sds_type",sds_type    
    #return None
    return data_SDS


def GetHeaderAttribut(headers,hdr_dscs,attribut_name):
    for i in range(len(headers)):
        if (hdr_dscs[i][0] == attribut_name):
            attribut=headers[i]
    return attribut


def GetRawData(file, tcfg, bigend):

    # Rcupration des descripteurs de headers
    tcfg['HEADERS'] = tcfg['HEADERS'].split()
    hdr_dscs = []
    for hdr in tcfg['HEADERS']:
        hdr_dscs.append(GetRecordDesc(hdr))

    # Rcupration du descripteur de record
    rec_dsc = GetRecordDesc(tcfg['RECORD'])
    #print rec_dsc
    # Ouverture du fichier
    f = open(tcfg['FILE'], "r")
    
    # Rcupration des donnes des headers
    headers = []
    for hdr_dsc in hdr_dscs:
        headers.append(GetRecord(f, hdr_dsc, bigend))


    scantime= []
    buff_dsc=[['buff1', 'b', 1, [6], 6],['scan_date', 'I', 4, [1], 1],['buff2', 'b', 1, [6], 6]]
    scantime.append(GetRecord(f, buff_dsc, bigend)[1])


    # Rcupration des donnes des records
    records = []
    records.append(GetRecord(f, rec_dsc, bigend))

    buff_dsc=[['buff1', 'b', 1, [10], 10],['scan_date', 'I', 4, [1], 1],['buff2', 'b', 1, [6], 6]]
    n=1
    try:
        #while True: rec_flds.append(GetRecord(f, rec_dsc, bigend))
        while True:
            records.append(GetRecord(f, rec_dsc, bigend))
            n=n+1
            if n%64 == 0 :
                scantime.append(GetRecord(f, buff_dsc, bigend)[1])
    except:
        f.close()
    		

    # Affichage des headers
    #for i in range(len(headers)):
    #    PrintRecord(headers[i], hdr_dscs[i])
    
    # Affichage du premier record
    #PrintRecord(records[0], rec_dsc)
    
    # Fabrication des donnes dsires
    #data = MakeData(headers,records,rec_dsc)
    
    
    return headers,hdr_dscs,records,rec_dsc,scantime

def ReadRawFile(tiroscfg):
    headers,hdr_dscs,records,rec_dsc,scantime=GetRawData(file, tiroscfg, tiroscfg['BIGEND'])
    return headers[0],hdr_dscs[0],records,rec_dsc,scantime






def GetFirstScanDateFile(tcfg):
    
    bigend = tcfg['BIGEND']
    
    # Rcupration des descripteurs de headers
    tcfg['HEADERS'] = tcfg['HEADERS'].split()
    hdr_dscs = []
    for hdr in tcfg['HEADERS']:
        hdr_dscs.append(GetRecordDesc(hdr))
    
    #Ouverture du fichier
    f = open(tcfg['FILE'], "r")
    
    # Rcupration des donnes des headers
    headers = []
    for hdr_dsc in hdr_dscs:
        headers.append(GetRecord(f, hdr_dsc, bigend))


    buff_dsc=[['buff1', 'b', 1, [6], 6],['scan_date', 'I', 4, [1], 1],['buff2', 'b', 1, [6], 6]]
    first_scan_date = GetRecord(f, buff_dsc, bigend)[1]


    f.close()
    
    return first_scan_date


