#!/usr/local/bin/python
# -*- coding: utf-8 -*-
###################################################################################################
###################################################################################################
##  Author :
##     CGTD-ICARE/UDEV Nicolas HENRIOT

##  License :
##     This file must be used under the terms of the CeCILL.
##     This source file is licensed as described in the file COPYING, which
##     you should have received as part of this distribution.  The terms
##     are also available at   http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
###################################################################################################
###################################################################################################
###################################################################################################

import numpy
import os
from datetime import date, timedelta, datetime

from dev_py.ConfigFile import *
from dev_py.ReadRawFile import *

nsd = numpy.int16(32767)

list_of_sds_SSMIS_env_BT = ['BT_19H','BT_19V','BT_22V','BT_37H','BT_37V']#['Channel13','Channel12','Channel14','Channel16','Channel15']
list_of_sds_SSMIS_env = ['Latitude_env','Longitude_env'] + list_of_sds_SSMIS_env_BT

list_of_sds_SSMIS_img_BT = ['BT_91H','BT_91V','BT_150H']#['Channel18','Channel17','Channel8']
list_of_sds_SSMIS_img = ['Latitude_img','Longitude_img'] + list_of_sds_SSMIS_img_BT

list_of_sds_SSMIS = ['Latitude_BF','Longitude_BF'] + list_of_sds_SSMIS_env_BT + ['Latitude_HF','Longitude_HF']+ list_of_sds_SSMIS_img_BT


from pyhdf import SD
import warnings
warnings.filterwarnings("ignore")
#-------------------------------------------------------------------------------------------------
class Writer:

    def __init__(self,data,sds_name,outfile,global_attribute):
        self.data     = data
        self.sds_name = sds_name
        self.outfile  = outfile
        self.global_attribute  = global_attribute


        # add the new dataset to out file
        self.write_sds_data(self.outfile,sds_name,data,global_attribute)


    def write_sds_data(self,filename,sds_name,data,global_attribute):
        """
        add a new dataset to the file "filename" named "sds_name" having the given data
        """
        # open the hdf file for writing
        sd = SD.SD(filename,SD.SDC.WRITE|SD.SDC.CREATE)
        hdf_type_code = self.numpy_2_hdftypecode(data.dtype)
        dataset = sd.create(sds_name,hdf_type_code,data.shape)

        #set dataset compression
        deflate_level=5
        SD.SDS.setcompress(dataset,SD.SDC.COMP_DEFLATE,deflate_level)

        #set dataset
        if data.ndim == 2:
            dataset[:,:] = data[:,:]
            #dim1 = dataset.dim(0)
            #dim2 = dataset.dim(1)
            #dim1.setname('nscan')
            #dim2.setname('npix')


        #set global attribute
        for key in global_attribute:
            setattr(sd,key,global_attribute[key])

        # Close dataset
        dataset.endaccess()

        # Close file
        sd.end()


    def numpy_2_hdftypecode(self,numpy_type_code):
        if numpy_type_code == numpy.float64:
            return SD.SDC.FLOAT64
        elif numpy_type_code == numpy.uint8:
            return SD.SDC.UINT8
        elif numpy_type_code == numpy.int16:
            return SD.SDC.INT16
        elif numpy_type_code == numpy.int32:
            return SD.SDC.INT32
        elif numpy_type_code == numpy.float32:
            return SD.SDC.FLOAT32

#-------------------------------------------------------------------------------------------------
def write_hdf_file(SSMIS_level1,hdf_outfile):

    global_attribute={}

    for key in list_of_sds_SSMIS:
       Writer(SSMIS_level1[key],key,hdf_outfile,global_attribute)

    return

#-------------------------------------------------------------------------------------------------   
def get_L1_SSMIS(cfg):


    cfg_SSMIS={}
    filein = cfg['input_file']


    cfg_SSMIS['FILE']            = filein
    cfg_SSMIS['BIGEND']          = 1
    cfg_SSMIS['HEADER_REV']      = os.path.join(cfg['FW_dir'],'conf/ssmis_header_rev.txt')
    cfg_SSMIS['HEADER_SCAN']     = os.path.join(cfg['FW_dir'],'conf/ssmis_header_scan.txt')
    cfg_SSMIS['RECORD_IMAGER']   = os.path.join(cfg['FW_dir'],'conf/ssmis_record_imager.txt')
    cfg_SSMIS['RECORD_ENV_ODD']  = os.path.join(cfg['FW_dir'],'conf/ssmis_record_env_odd.txt')
    cfg_SSMIS['RECORD_ENV_EVEN'] = os.path.join(cfg['FW_dir'],'conf/ssmis_record_env_even.txt')
    cfg_SSMIS['RECORD_LAS']      = os.path.join(cfg['FW_dir'],'conf/ssmis_record_LAS.txt')
    cfg_SSMIS['RECORD_UAS']      = os.path.join(cfg['FW_dir'],'conf/ssmis_record_UAS.txt')



    bigend = cfg_SSMIS['BIGEND']

    ##############################img = High et env = Low####################
    # Ouverture du fichier
    f = open(cfg_SSMIS['FILE'], "r")

    cfg_SSMIS['HEADER_REV'] = cfg_SSMIS['HEADER_REV'].split()
    hdr_dscs = []
    for hdr in cfg_SSMIS['HEADER_REV']:
        hdr_dscs.append(GetRecordDesc(hdr))
        
    headers_rev = []
    for hdr_dsc in hdr_dscs:
        headers_rev.append(GetRecord(f, hdr_dsc, bigend))
        
    num_of_rec = MakeData(headers_rev,hdr_dsc,'num_SDR')[0][0]

    proc_sta =  MakeData(headers_rev,hdr_dsc,'proc_sta')[0][0]
    if proc_sta < 0:
        slope = 100.
    else:
        slope = 10.
    
    cfg_SSMIS['HEADER_SCAN'] = cfg_SSMIS['HEADER_SCAN'].split()           
    hdr_dscs = []
    for hdr in cfg_SSMIS['HEADER_SCAN']:
        hdr_dscs.append(GetRecordDesc(hdr))

    rec_dsc_img      = GetRecordDesc(cfg_SSMIS['RECORD_IMAGER'])
    rec_dsc_env_odd  = GetRecordDesc(cfg_SSMIS['RECORD_ENV_ODD'])
    rec_dsc_env_even = GetRecordDesc(cfg_SSMIS['RECORD_ENV_EVEN'])
    rec_dsc_las      = GetRecordDesc(cfg_SSMIS['RECORD_LAS'])
    rec_dsc_uas      = GetRecordDesc(cfg_SSMIS['RECORD_UAS'])
    
    nb_pix_per_scan_img_max = 180
    nb_pix_per_scan_env_max = 90
    nb_pix_per_scan_las_max = 60
    nb_pix_per_scan_uas_max = 30
    
    nb_byte_headers_scan    = 360
    nb_byte_record_imager   = 20
    nb_byte_record_env_odd  = 36
    nb_byte_record_env_even = 18
    nb_byte_record_LAS      = 40
    nb_byte_record_UAS      = 28
    nb_byte_per_block       = 512
    
    records_img = []
    records_env_odd = []
    records_env_even = []
    records_las = []
    records_uas = []

    year_scan_env   = []
    day_scan_env    = []
    hour_scan_env   = []
    minute_scan_env = []
    scan_time_env   = []
    
    year_scan_img   = []
    day_scan_img    = []
    hour_scan_img   = []
    minute_scan_img = []
    scan_time_img   = []
   
    for rec in range(num_of_rec):
        
        headers_scan = []
        for hdr_dsc in hdr_dscs:
            headers_scan.append(GetRecord(f, hdr_dsc, bigend))
            
        num_of_scan_img = MakeData(headers_scan,hdr_dsc,'Img_scans')[0][0]
        num_of_scan_env = MakeData(headers_scan,hdr_dsc,'Env_scans')[0][0]
        num_of_scan_las = MakeData(headers_scan,hdr_dsc,'LAS_scans')[0][0]       
        num_of_scan_uas = MakeData(headers_scan,hdr_dsc,'UAS_scans')[0][0]
        
        nb_pix_per_scan_img = MakeData(headers_scan,hdr_dsc,'Img_nb_pix')[0]
        nb_pix_per_scan_env = MakeData(headers_scan,hdr_dsc,'Env_nb_pix')[0]
        nb_pix_per_scan_las = MakeData(headers_scan,hdr_dsc,'LAS_nb_pix')[0]
        nb_pix_per_scan_uas = MakeData(headers_scan,hdr_dsc,'UAS_nb_pix')[0]
     
        nb_pix_img = 0
        
        for i in xrange(num_of_scan_img):
            for j in xrange(nb_pix_per_scan_img[i]):
                year_scan_img.append(headers_scan[0][1])
                day_scan_img.append(headers_scan[0][2])
                scan_time_img.append(headers_scan[0][10+i])
                records_img.append(GetRecord(f, rec_dsc_img, bigend))
            nb_pix_img = nb_pix_img + nb_pix_per_scan_img[i]
 

        nb_pix_env_odd  = 0
        nb_pix_env_even = 0
        for i in xrange(num_of_scan_env):

            if i%2 == 0 :
                if nb_pix_per_scan_env[i] == 0:
                    for j in xrange(nb_pix_per_scan_env_max):
                        records_env_odd.append([nsd, nsd, nsd, nsd, nsd, nsd, nsd, nsd, nsd, nsd, nsd, nsd, nsd, nsd, nsd, nsd, nsd, nsd, \
                                                array([nsd, nsd, nsd, nsd])])
                else:
                    for j in xrange(nb_pix_per_scan_env[i]):
                        year_scan_env.append(headers_scan[0][1])
                        day_scan_env.append(headers_scan[0][2])
                        scan_time_env.append(headers_scan[0][39+i])
                        records_env_odd.append(GetRecord(f, rec_dsc_env_odd, bigend))
                    nb_pix_env_odd = nb_pix_env_odd + nb_pix_per_scan_env[i]
            else:
                if nb_pix_per_scan_env[i] == 0:
                    for j in xrange(nb_pix_per_scan_env_max):
                        records_env_even.append([nsd, nsd, nsd, nsd, nsd, nsd, nsd, nsd, nsd, nsd])
                else:        
                    for j in xrange(nb_pix_per_scan_env[i]):
                        year_scan_env.append(headers_scan[0][1])
                        day_scan_env.append(headers_scan[0][2])
                        scan_time_env.append(headers_scan[0][39+i])
                        records_env_even.append(GetRecord(f, rec_dsc_env_even, bigend))
                    nb_pix_env_even = nb_pix_env_even + nb_pix_per_scan_env[i]

            
        buff = nb_pix_per_scan_las.sum() * nb_byte_record_LAS        
        buff_dsc=[['buff_data', 'b', 1, [buff], buff]]
        GetRecord(f, buff_dsc, bigend)
        
        buff = nb_pix_per_scan_uas.sum() * nb_byte_record_UAS        
        buff_dsc=[['buff_data', 'b', 1, [buff], buff]]
        GetRecord(f, buff_dsc, bigend)
        
        buff = nb_byte_per_block - (nb_byte_headers_scan + \
                                    nb_pix_img                  * nb_byte_record_imager + \
                                    nb_pix_env_odd              * nb_byte_record_env_odd + \
                                    nb_pix_env_even             * nb_byte_record_env_even + \
                                    nb_pix_per_scan_las.sum()   * nb_byte_record_LAS + \
                                    nb_pix_per_scan_uas.sum()   * nb_byte_record_UAS)%nb_byte_per_block            
        if buff%512 != 0 :            
            buff_dsc=[['buff_data', 'b', 1, [buff], buff]]            
            GetRecord(f, buff_dsc, bigend)
            
        
    L1={}
    
    for sds_name in list_of_sds_SSMIS_env:
        data_SDS_env_odd = MakeData(records_env_odd,rec_dsc_env_odd,sds_name)
        l,c = data_SDS_env_odd.shape
        nb_scan_odd = l/nb_pix_per_scan_env_max
        data_SDS_env_odd.shape = (nb_scan_odd,nb_pix_per_scan_env_max)
        
        data_SDS_env_even = MakeData(records_env_even,rec_dsc_env_even,sds_name)
        l,c = data_SDS_env_even.shape
        nb_scan_even = l/nb_pix_per_scan_env_max
        data_SDS_env_even.shape = (nb_scan_even,nb_pix_per_scan_env_max)
        
        nb_scan = nb_scan_odd + nb_scan_even
        
        data_temp         = (numpy.zeros((nb_scan,nb_pix_per_scan_env_max),dtype=numpy.int16)).copy()
        data_temp[:,:]    = nsd
        data_temp[::2,:]  = data_SDS_env_odd
        data_temp[1::2,:] = data_SDS_env_even
        cond = (data_temp[:,:] != nsd)
        
        if sds_name in list_of_sds_SSMIS_env_BT:
            offset = 27315
        else:
            offset = 0            
        L1[sds_name] = ((data_temp[cond]).copy() + offset).astype(numpy.float32) / slope

        nb_scan_env = (L1[sds_name].size)/nb_pix_per_scan_env_max
        (L1[sds_name]).shape = (nb_scan_env,nb_pix_per_scan_env_max)

            
    for sds_name in list_of_sds_SSMIS_img:
        data_SDS_img = MakeData(records_img,rec_dsc_img,sds_name)
        l,c = data_SDS_img.shape
        nb_scan_img = l/nb_pix_per_scan_img_max
        
        data_SDS_img.shape = (nb_scan_img,nb_pix_per_scan_img_max)
        
        if sds_name in list_of_sds_SSMIS_img_BT:
            offset = 27315
        else:
            offset = 0
        L1[sds_name] = ((data_SDS_img[:nb_scan_env,:]).copy() + offset).astype(numpy.float32) / slope
        
    
    L1['nb_pix_per_scan'] = nb_pix_per_scan_env_max
    L1['nb_scan'] = nb_scan_env
    
    L1['nb_pix_per_scan_img'] = nb_pix_per_scan_img_max
    L1['nb_scan_img'] = nb_scan_img
    

    year_img           = numpy.array(year_scan_img)
    day_img            = numpy.array(day_scan_img)
    scantime_img       = numpy.array(scan_time_img)
    year_img.shape     = (nb_scan_img,nb_pix_per_scan_img_max)
    day_img.shape      = (nb_scan_img,nb_pix_per_scan_img_max)
    scantime_img.shape = (nb_scan_img,nb_pix_per_scan_img_max)
    year_h             = year_img[:nb_scan_env,0]
    days_h             = day_img[:nb_scan_env,0]
    scantime_h         = scantime_img[:nb_scan_env,0]

    L1['year_h']   = (numpy.zeros((L1['nb_scan']),dtype=numpy.float32)).copy()
    L1['month_h']  = (numpy.zeros((L1['nb_scan']),dtype=numpy.float32)).copy()
    L1['days_h']   = (numpy.zeros((L1['nb_scan']),dtype=numpy.float32)).copy()
    L1['day_h']    = (numpy.zeros((L1['nb_scan']),dtype=numpy.float32)).copy()
    L1['hour_h']   = (numpy.zeros((L1['nb_scan']),dtype=numpy.float32)).copy()
    L1['minute_h'] = (numpy.zeros((L1['nb_scan']),dtype=numpy.float32)).copy()
    L1['second_h'] = (numpy.zeros((L1['nb_scan']),dtype=numpy.float32)).copy()

    for i in xrange(L1['nb_scan']):
        d = timedelta(days=numpy.float(days_h[i])) + \
            timedelta(milliseconds=numpy.float(scantime_h[i])) + \
            datetime(year_h[i],1,1,0,0,0) - timedelta(days=1)
        
        L1['year_h'][i]   = d.year
        L1['month_h'][i]  = d.month
        L1['day_h'][i]    = d.day
        L1['hour_h'][i]   = d.hour
        L1['minute_h'][i] = d.minute
        L1['second_h'][i] = d.second
        
        L1['days_h'][i]   = days_h[i]
       
    L1['scantime_h'] = ((numpy.array(scantime_h[:])).copy()).astype(numpy.float32)


    
    year_env           = numpy.array(year_scan_env)
    day_env            = numpy.array(day_scan_env)
    scantime_env       = numpy.array(scan_time_env)
    year_env.shape     = (nb_scan_env,nb_pix_per_scan_env_max)
    day_env.shape      = (nb_scan_env,nb_pix_per_scan_env_max)
    scantime_env.shape = (nb_scan_env,nb_pix_per_scan_env_max)
    year_l             = year_env[:nb_scan_env,0]
    days_l             = day_env[:nb_scan_env,0]
    scantime_l         = scantime_env[:nb_scan_env,0]
       
    L1['year_l']   = (numpy.zeros((L1['nb_scan']),dtype=numpy.float32)).copy()
    L1['month_l']  = (numpy.zeros((L1['nb_scan']),dtype=numpy.float32)).copy()
    L1['days_l']   = (numpy.zeros((L1['nb_scan']),dtype=numpy.float32)).copy()
    L1['day_l']    = (numpy.zeros((L1['nb_scan']),dtype=numpy.float32)).copy()
    L1['hour_l']   = (numpy.zeros((L1['nb_scan']),dtype=numpy.float32)).copy()
    L1['minute_l'] = (numpy.zeros((L1['nb_scan']),dtype=numpy.float32)).copy()
    L1['second_l'] = (numpy.zeros((L1['nb_scan']),dtype=numpy.float32)).copy()
       

    for i in xrange(L1['nb_scan']):
        d = timedelta(days=numpy.float(days_l[i])) + \
            timedelta(milliseconds=numpy.float(scantime_l[i])) + \
            datetime(year_l[i],1,1,0,0,0) - timedelta(days=1)

        L1['year_l'][i]   = d.year
        L1['month_l'][i]  = d.month
        L1['day_l'][i]    = d.day
        L1['hour_l'][i]   = d.hour
        L1['minute_l'][i] = d.minute
        L1['second_l'][i] = d.second

        L1['days_l'][i]   = days_l[i]
        
    L1['scantime_l'] = ((numpy.array(scantime_l[:])).copy()).astype(numpy.float32)
    
    L1['Latitude_BF']  =  (L1['Latitude_env']).copy()
    L1['Longitude_BF'] =  (L1['Longitude_env']).copy()
    L1['Latitude_HF']  =  (L1['Latitude_img']).copy()
    L1['Longitude_HF'] =  (L1['Longitude_img']).copy()

    return L1
#-------------------------------------------------------------------------------------------------   
def VerifCfg(cfg):
    ok, msg = (1, "")
    if (len(cfg['FW_dir']) == 0 or
        len(cfg['input_file']) == 0 or
        len(cfg['output_dir']) == 0 ) :
        ok = 0
        msg += 'no value parameter \n'
    return (ok, msg)

#-------------------------------------------------------------------------------------------------
#-------------------------------------------------------------------------------------------------   

def main():
    # Parameters definition
    #    name       type  option default
    req_cfg = {
       'FW_dir'                      : (  str   , 'f:', None),
       'input_file'                  : (  str   , 'i:', None),
       'output_dir'                  : (  str   , 'o:', None),
        }

    # Settings
    try:
        config = Config(VerifCfg, [""])
        cfg = config.get(req_cfg)
        
    except ConfigError, e:
        print "Configuration error; aborting..."
        return

    hdf_outfile = os.path.basename(cfg['input_file'])[:-3]+'.hdf'

    L1_SSMIS = get_L1_SSMIS(cfg)
    write_hdf_file(L1_SSMIS,os.path.join(cfg['output_dir'],hdf_outfile))
    
if __name__== "__main__":
    main()

